@@ -33,12 +33,12 @@
 
 #include "wine/debug.h"
 
-static IDirectInputDeviceImpl *impl_from_IDirectInputDevice8A( IDirectInputDevice8A *iface )
+static struct dinput_device *impl_from_IDirectInputDevice8A( IDirectInputDevice8A *iface )
 {
-    return CONTAINING_RECORD( iface, IDirectInputDeviceImpl, IDirectInputDevice8A_iface );
+    return CONTAINING_RECORD( iface, struct dinput_device, IDirectInputDevice8A_iface );
 }
 
-static IDirectInputDevice8W *IDirectInputDevice8W_from_impl( IDirectInputDeviceImpl *impl )
+static IDirectInputDevice8W *IDirectInputDevice8W_from_impl( struct dinput_device *impl )
 {
     return &impl->IDirectInputDevice8W_iface;
 }
@@ -46,7 +46,7 @@ static IDirectInputDevice8W *IDirectInputDevice8W_from_impl( IDirectInputDeviceI
 static inline IDirectInputDevice8A *IDirectInputDevice8A_from_IDirectInputDevice8W( IDirectInputDevice8W *iface )
 {
     if (!iface) return NULL;
-    return &CONTAINING_RECORD( iface, IDirectInputDeviceImpl, IDirectInputDevice8W_iface )->IDirectInputDevice8A_iface;
+    return &CONTAINING_RECORD( iface, struct dinput_device, IDirectInputDevice8W_iface )->IDirectInputDevice8A_iface;
 }
 
 static inline IDirectInputDeviceA *IDirectInputDeviceA_from_IDirectInputDeviceW( IDirectInputDeviceW *iface )
@@ -269,28 +269,28 @@ static HRESULT diconfiguredevicesparams_atow( const DICONFIGUREDEVICESPARAMSA *i
 
 static HRESULT WINAPI dinput_device_a_QueryInterface( IDirectInputDevice8A *iface_a, REFIID iid, void **out )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_QueryInterface( iface_w, iid, out );
 }
 
 static ULONG WINAPI dinput_device_a_AddRef( IDirectInputDevice8A *iface_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_AddRef( iface_w );
 }
 
 static ULONG WINAPI dinput_device_a_Release( IDirectInputDevice8A *iface_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Release( iface_w );
 }
 
 static HRESULT WINAPI dinput_device_a_GetCapabilities( IDirectInputDevice8A *iface_a, DIDEVCAPS *caps )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_GetCapabilities( iface_w, caps );
 }
@@ -314,7 +314,7 @@ static HRESULT WINAPI dinput_device_a_EnumObjects( IDirectInputDevice8A *iface_a
                                                    void *ref, DWORD flags )
 {
     struct enum_objects_wtoa_params params = {callback, ref};
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
 
     if (!callback) return DIERR_INVALIDPARAM;
@@ -324,35 +324,35 @@ static HRESULT WINAPI dinput_device_a_EnumObjects( IDirectInputDevice8A *iface_a
 
 static HRESULT WINAPI dinput_device_a_GetProperty( IDirectInputDevice8A *iface_a, REFGUID guid, DIPROPHEADER *header )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_GetProperty( iface_w, guid, header );
 }
 
 static HRESULT WINAPI dinput_device_a_SetProperty( IDirectInputDevice8A *iface_a, REFGUID guid, const DIPROPHEADER *header )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SetProperty( iface_w, guid, header );
 }
 
 static HRESULT WINAPI dinput_device_a_Acquire( IDirectInputDevice8A *iface_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Acquire( iface_w );
 }
 
 static HRESULT WINAPI dinput_device_a_Unacquire( IDirectInputDevice8A *iface_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Unacquire( iface_w );
 }
 
 static HRESULT WINAPI dinput_device_a_GetDeviceState( IDirectInputDevice8A *iface_a, DWORD count, void *data )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_GetDeviceState( iface_w, count, data );
 }
@@ -360,28 +360,28 @@ static HRESULT WINAPI dinput_device_a_GetDeviceState( IDirectInputDevice8A *ifac
 static HRESULT WINAPI dinput_device_a_GetDeviceData( IDirectInputDevice8A *iface_a, DWORD data_size, DIDEVICEOBJECTDATA *data,
                                                      DWORD *entries, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_GetDeviceData( iface_w, data_size, data, entries, flags );
 }
 
 static HRESULT WINAPI dinput_device_a_SetDataFormat( IDirectInputDevice8A *iface_a, const DIDATAFORMAT *format )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SetDataFormat( iface_w, format );
 }
 
 static HRESULT WINAPI dinput_device_a_SetEventNotification( IDirectInputDevice8A *iface_a, HANDLE event )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SetEventNotification( iface_w, event );
 }
 
 static HRESULT WINAPI dinput_device_a_SetCooperativeLevel( IDirectInputDevice8A *iface_a, HWND window, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SetCooperativeLevel( iface_w, window, flags );
 }
@@ -389,7 +389,7 @@ static HRESULT WINAPI dinput_device_a_SetCooperativeLevel( IDirectInputDevice8A
 static HRESULT WINAPI dinput_device_a_GetObjectInfo( IDirectInputDevice8A *iface_a, DIDEVICEOBJECTINSTANCEA *instance_a,
                                                      DWORD obj, DWORD how )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIDEVICEOBJECTINSTANCEW instance_w = {sizeof(instance_w)};
     HRESULT hr;
@@ -407,7 +407,7 @@ static HRESULT WINAPI dinput_device_a_GetObjectInfo( IDirectInputDevice8A *iface
 
 static HRESULT WINAPI dinput_device_a_GetDeviceInfo( IDirectInputDevice8A *iface_a, DIDEVICEINSTANCEA *instance_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIDEVICEINSTANCEW instance_w = {sizeof(instance_w)};
     HRESULT hr;
@@ -424,14 +424,14 @@ static HRESULT WINAPI dinput_device_a_GetDeviceInfo( IDirectInputDevice8A *iface
 
 static HRESULT WINAPI dinput_device_a_RunControlPanel( IDirectInputDevice8A *iface_a, HWND owner, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_RunControlPanel( iface_w, owner, flags );
 }
 
 static HRESULT WINAPI dinput_device_a_Initialize( IDirectInputDevice8A *iface_a, HINSTANCE instance, DWORD version, REFGUID guid )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Initialize( iface_w, instance, version, guid );
 }
@@ -439,7 +439,7 @@ static HRESULT WINAPI dinput_device_a_Initialize( IDirectInputDevice8A *iface_a,
 static HRESULT WINAPI dinput_device_a_CreateEffect( IDirectInputDevice8A *iface_a, REFGUID guid, const DIEFFECT *effect,
                                                     IDirectInputEffect **out, IUnknown *outer )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_CreateEffect( iface_w, guid, effect, out, outer );
 }
@@ -463,7 +463,7 @@ static HRESULT WINAPI dinput_device_a_EnumEffects( IDirectInputDevice8A *iface_a
                                                    void *ref, DWORD type )
 {
     struct enum_effects_wtoa_params params = {callback, ref};
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
 
     if (!callback) return DIERR_INVALIDPARAM;
@@ -473,7 +473,7 @@ static HRESULT WINAPI dinput_device_a_EnumEffects( IDirectInputDevice8A *iface_a
 
 static HRESULT WINAPI dinput_device_a_GetEffectInfo( IDirectInputDevice8A *iface_a, DIEFFECTINFOA *info_a, REFGUID guid )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIEFFECTINFOW info_w = {sizeof(info_w)};
     HRESULT hr;
@@ -489,14 +489,14 @@ static HRESULT WINAPI dinput_device_a_GetEffectInfo( IDirectInputDevice8A *iface
 
 static HRESULT WINAPI dinput_device_a_GetForceFeedbackState( IDirectInputDevice8A *iface_a, DWORD *state )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_GetForceFeedbackState( iface_w, state );
 }
 
 static HRESULT WINAPI dinput_device_a_SendForceFeedbackCommand( IDirectInputDevice8A *iface_a, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SendForceFeedbackCommand( iface_w, flags );
 }
@@ -504,21 +504,21 @@ static HRESULT WINAPI dinput_device_a_SendForceFeedbackCommand( IDirectInputDevi
 static HRESULT WINAPI dinput_device_a_EnumCreatedEffectObjects( IDirectInputDevice8A *iface_a, LPDIENUMCREATEDEFFECTOBJECTSCALLBACK callback,
                                                                 void *ref, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_EnumCreatedEffectObjects( iface_w, callback, ref, flags );
 }
 
 static HRESULT WINAPI dinput_device_a_Escape( IDirectInputDevice8A *iface_a, DIEFFESCAPE *escape )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Escape( iface_w, escape );
 }
 
 static HRESULT WINAPI dinput_device_a_Poll( IDirectInputDevice8A *iface_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_Poll( iface_w );
 }
@@ -526,7 +526,7 @@ static HRESULT WINAPI dinput_device_a_Poll( IDirectInputDevice8A *iface_a )
 static HRESULT WINAPI dinput_device_a_SendDeviceData( IDirectInputDevice8A *iface_a, DWORD count, const DIDEVICEOBJECTDATA *data,
                                                       DWORD *inout, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     return IDirectInputDevice8_SendDeviceData( iface_w, count, data, inout, flags );
 }
@@ -534,7 +534,7 @@ static HRESULT WINAPI dinput_device_a_SendDeviceData( IDirectInputDevice8A *ifac
 static HRESULT WINAPI dinput_device_a_EnumEffectsInFile( IDirectInputDevice8A *iface_a, const char *filename_a, LPDIENUMEFFECTSINFILECALLBACK callback,
                                                          void *ref, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     WCHAR buffer[MAX_PATH], *filename_w = buffer;
 
@@ -547,7 +547,7 @@ static HRESULT WINAPI dinput_device_a_EnumEffectsInFile( IDirectInputDevice8A *i
 static HRESULT WINAPI dinput_device_a_WriteEffectToFile( IDirectInputDevice8A *iface_a, const char *filename_a, DWORD entries,
                                                          DIFILEEFFECT *file_effect, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     WCHAR buffer[MAX_PATH], *filename_w = buffer;
 
@@ -560,7 +560,7 @@ static HRESULT WINAPI dinput_device_a_WriteEffectToFile( IDirectInputDevice8A *i
 static HRESULT WINAPI dinput_device_a_BuildActionMap( IDirectInputDevice8A *iface_a, DIACTIONFORMATA *format_a,
                                                       const char *username_a, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIACTIONFORMATW format_w = {sizeof(format_w), sizeof(DIACTIONW)};
     HRESULT hr;
@@ -589,7 +589,7 @@ static HRESULT WINAPI dinput_device_a_BuildActionMap( IDirectInputDevice8A *ifac
 static HRESULT WINAPI dinput_device_a_SetActionMap( IDirectInputDevice8A *iface_a, DIACTIONFORMATA *format_a,
                                                     const char *username_a, DWORD flags )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIACTIONFORMATW format_w = {sizeof(format_w), sizeof(DIACTIONW)};
     HRESULT hr;
@@ -617,7 +617,7 @@ static HRESULT WINAPI dinput_device_a_SetActionMap( IDirectInputDevice8A *iface_
 
 static HRESULT WINAPI dinput_device_a_GetImageInfo( IDirectInputDevice8A *iface_a, DIDEVICEIMAGEINFOHEADERA *header_a )
 {
-    IDirectInputDeviceImpl *impl = impl_from_IDirectInputDevice8A( iface_a );
+    struct dinput_device *impl = impl_from_IDirectInputDevice8A( iface_a );
     IDirectInputDevice8W *iface_w = IDirectInputDevice8W_from_impl( impl );
     DIDEVICEIMAGEINFOHEADERW header_w = {sizeof(header_w), sizeof(DIDEVICEIMAGEINFOW)};
     HRESULT hr;
