@@ -28,25 +28,6 @@
 #include "wine/list.h"
 #include "dinput_private.h"
 
-typedef struct
-{
-    int size;
-    int offset_in;
-    int offset_out;
-    int value;
-} DataTransform;
-
-typedef struct
-{
-    int                         size;
-    int                         internal_format_size;
-    DataTransform              *dt;
-
-    int                        *offsets;     /* object offsets */
-    LPDIDATAFORMAT              wine_df;     /* wine internal data format */
-    LPDIDATAFORMAT              user_df;     /* user defined data format */
-} DataFormat;
-
 typedef struct
 {
     unsigned int offset;
@@ -63,21 +44,33 @@ struct dinput_device_vtbl
     HRESULT (*enum_objects)( IDirectInputDevice8W *iface, const DIPROPHEADER *filter, DWORD flags,
                              LPDIENUMDEVICEOBJECTSCALLBACKW callback, void *context );
     HRESULT (*get_property)( IDirectInputDevice8W *iface, DWORD property, DIPROPHEADER *header,
-                             DIDEVICEOBJECTINSTANCEW *instance );
-    HRESULT (*set_property)( IDirectInputDevice8W *iface, DWORD property, const DIPROPHEADER *header,
                              const DIDEVICEOBJECTINSTANCEW *instance );
     HRESULT (*get_effect_info)( IDirectInputDevice8W *iface, DIEFFECTINFOW *info, const GUID *guid );
     HRESULT (*create_effect)( IDirectInputDevice8W *iface, IDirectInputEffect **out );
-    HRESULT (*send_force_feedback_command)( IDirectInputDevice8W *iface, DWORD command );
+    HRESULT (*send_force_feedback_command)( IDirectInputDevice8W *iface, DWORD command, BOOL unacquire );
+    HRESULT (*send_device_gain)( IDirectInputDevice8W *iface, LONG device_gain );
     HRESULT (*enum_created_effect_objects)( IDirectInputDevice8W *iface, LPDIENUMCREATEDEFFECTOBJECTSCALLBACK callback,
                                             void *context, DWORD flags );
 };
 
 #define DEVICE_STATE_MAX_SIZE 1024
 
+struct object_properties
+{
+    LONG bit_size;
+    LONG physical_min;
+    LONG physical_max;
+    LONG logical_min;
+    LONG logical_max;
+    LONG range_min;
+    LONG range_max;
+    LONG deadzone;
+    LONG saturation;
+    DWORD calibration_mode;
+};
+
 /* Device implementation */
-typedef struct IDirectInputDeviceImpl IDirectInputDeviceImpl;
-struct IDirectInputDeviceImpl
+struct dinput_device
 {
     IDirectInputDevice8W        IDirectInputDevice8W_iface;
     IDirectInputDevice8A        IDirectInputDevice8A_iface;
@@ -103,7 +96,8 @@ struct IDirectInputDeviceImpl
     BOOL                        overflow;    /* return DI_BUFFEROVERFLOW in 'GetDeviceData' */
     DWORD                       buffersize;  /* size of the queue - set in 'SetProperty'    */
 
-    DataFormat                  data_format; /* user data format and wine to user format converter */
+    DIDATAFORMAT *device_format;
+    DIDATAFORMAT *user_format;
 
     /* Action mapping */
     int                         num_actions; /* number of actions mapped */
@@ -115,12 +109,16 @@ struct IDirectInputDeviceImpl
 
     BYTE device_state_report_id;
     BYTE device_state[DEVICE_STATE_MAX_SIZE];
+
+    BOOL autocenter;
+    LONG device_gain;
+    struct object_properties *object_properties;
 };
 
-extern HRESULT direct_input_device_alloc( SIZE_T size, const struct dinput_device_vtbl *vtbl,
-                                          const GUID *guid, IDirectInputImpl *dinput, void **out ) DECLSPEC_HIDDEN;
-extern HRESULT direct_input_device_init( IDirectInputDevice8W *iface );
-extern void direct_input_device_destroy( IDirectInputDevice8W *iface );
+extern HRESULT dinput_device_alloc( SIZE_T size, const struct dinput_device_vtbl *vtbl, const GUID *guid,
+                                    IDirectInputImpl *dinput, void **out ) DECLSPEC_HIDDEN;
+extern HRESULT dinput_device_init( IDirectInputDevice8W *iface );
+extern void dinput_device_destroy( IDirectInputDevice8W *iface );
 
 extern BOOL get_app_key(HKEY*, HKEY*) DECLSPEC_HIDDEN;
 extern DWORD get_config_key( HKEY, HKEY, const WCHAR *, WCHAR *, DWORD ) DECLSPEC_HIDDEN;
