@@ -25,6 +25,7 @@
 #include "mferror.h"
 
 #include "evr_classes.h"
+#include "evr_private.h"
 
 #include "initguid.h"
 #include "evr9.h"
@@ -34,7 +35,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(evr);
 
-#define MAX_MIXER_INPUT_STREAMS 16
+#define MAX_MIXER_INPUT_SUBSTREAMS (15)
+#define MAX_MIXER_INPUT_STREAMS (MAX_MIXER_INPUT_SUBSTREAMS + 1)
 
 struct input_stream
 {
@@ -43,6 +45,7 @@ struct input_stream
     IMFMediaType *media_type;
     MFVideoNormalizedRect rect;
     unsigned int zorder;
+    SIZE frame_size;
     IMFSample *sample;
     unsigned int sample_requested : 1;
 };
@@ -80,6 +83,7 @@ struct video_mixer
 
     struct input_stream inputs[MAX_MIXER_INPUT_STREAMS];
     unsigned int input_ids[MAX_MIXER_INPUT_STREAMS];
+    struct input_stream *zorder[MAX_MIXER_INPUT_STREAMS];
     unsigned int input_count;
     struct output_stream output;
 
@@ -91,8 +95,14 @@ struct video_mixer
     IMFAttributes *attributes;
     IMFAttributes *internal_attributes;
     unsigned int mixing_flags;
-    unsigned int is_streaming;
-    COLORREF bkgnd_color;
+    unsigned int is_streaming : 1;
+    unsigned int output_rendered : 1;
+    struct
+    {
+        COLORREF rgba;
+        DXVA2_AYUVSample16 ayuv;
+    } bkgnd_color;
+    MFVideoArea aperture;
     LONGLONG lower_bound;
     LONGLONG upper_bound;
     CRITICAL_SECTION cs;
@@ -182,6 +192,36 @@ static void video_mixer_init_input(struct input_stream *stream)
     stream->rect.right = stream->rect.bottom = 1.0f;
 }
 
+static int __cdecl video_mixer_zorder_sort_compare(const void *a, const void *b)
+{
+    const struct input_stream *left = *(void **)a, *right = *(void **)b;
+    return left->zorder != right->zorder ? (left->zorder < right->zorder ? -1 : 1) : 0;
+};
+
+static void video_mixer_update_zorder_map(struct video_mixer *mixer)
+{
+    unsigned int i;
+
+    for (i = 0; i < mixer->input_count; ++i)
+        mixer->zorder[i] = &mixer->inputs[i];
+
+    qsort(mixer->zorder, mixer->input_count, sizeof(*mixer->zorder), video_mixer_zorder_sort_compare);
+}
+
+static void video_mixer_flush_input(struct video_mixer *mixer)
+{
+    unsigned int i;
+
+    for (i = 0; i < mixer->input_count; ++i)
+    {
+        if (mixer->inputs[i].sample)
+            IMFSample_Release(mixer->inputs[i].sample);
+        mixer->inputs[i].sample = NULL;
+        mixer->inputs[i].sample_requested = 0;
+    }
+    mixer->output_rendered = 0;
+}
+
 static void video_mixer_clear_types(struct video_mixer *mixer)
 {
     unsigned int i;
@@ -191,10 +231,8 @@ static void video_mixer_clear_types(struct video_mixer *mixer)
         if (mixer->inputs[i].media_type)
             IMFMediaType_Release(mixer->inputs[i].media_type);
         mixer->inputs[i].media_type = NULL;
-        if (mixer->inputs[i].sample)
-            IMFSample_Release(mixer->inputs[i].sample);
-        mixer->inputs[i].sample = NULL;
     }
+    video_mixer_flush_input(mixer);
     for (i = 0; i < mixer->output.rt_formats_count; ++i)
     {
         IMFMediaType_Release(mixer->output.rt_formats[i].media_type);
@@ -500,6 +538,7 @@ static HRESULT WINAPI video_mixer_transform_DeleteInputStream(IMFTransform *ifac
             memmove(&mixer->inputs[idx], &mixer->inputs[idx + 1], (mixer->input_count - idx) * sizeof(*mixer->inputs));
             memmove(&mixer->input_ids[idx], &mixer->input_ids[idx + 1], (mixer->input_count - idx) *
                     sizeof(*mixer->input_ids));
+            video_mixer_update_zorder_map(mixer);
         }
     }
 
@@ -569,6 +608,8 @@ static HRESULT WINAPI video_mixer_transform_AddInputStreams(IMFTransform *iface,
                     input->zorder = zorder;
                 zorder++;
             }
+
+            video_mixer_update_zorder_map(mixer);
         }
     }
     LeaveCriticalSection(&mixer->cs);
@@ -603,8 +644,8 @@ static HRESULT WINAPI video_mixer_transform_GetOutputAvailableType(IMFTransform
         hr = MF_E_NO_MORE_TYPES;
     else
     {
-        *type = mixer->output.rt_formats[index].media_type;
-        IMFMediaType_AddRef(*type);
+        if (SUCCEEDED(hr = MFCreateMediaType(type)))
+            hr = IMFMediaType_CopyAllItems(mixer->output.rt_formats[index].media_type, (IMFAttributes *)*type);
     }
 
     LeaveCriticalSection(&mixer->cs);
@@ -642,21 +683,58 @@ done:
     return hr;
 }
 
-static int __cdecl rt_formats_sort_compare(const void *left, const void *right)
+static void video_mixer_append_rt_format(struct rt_format *rt_formats, unsigned int *count,
+        const GUID *device, D3DFORMAT format)
 {
-    const struct rt_format *format1 = left, *format2 = right;
+    unsigned int i;
 
-    if (format1->format < format2->format) return -1;
-    if (format1->format > format2->format) return 1;
-    return 0;
+    for (i = 0; i < *count; ++i)
+    {
+        if (rt_formats[i].format == format) return;
+    }
+
+    rt_formats[*count].format = format;
+    rt_formats[*count].device = *device;
+    *count += 1;
+}
+
+static unsigned int video_mixer_get_interlace_mode_from_video_desc(const DXVA2_VideoDesc *video_desc)
+{
+    switch (video_desc->SampleFormat.SampleFormat)
+    {
+        case DXVA2_SampleFieldInterleavedEvenFirst:
+            return MFVideoInterlace_FieldInterleavedUpperFirst;
+        case DXVA2_SampleFieldInterleavedOddFirst:
+            return MFVideoInterlace_FieldInterleavedLowerFirst;
+        case DXVA2_SampleFieldSingleEven:
+            return MFVideoInterlace_FieldSingleUpper;
+        case DXVA2_SampleFieldSingleOdd:
+            return MFVideoInterlace_FieldSingleLower;
+        default:
+            return MFVideoInterlace_Progressive;
+    }
+}
+
+static void mf_get_attribute_uint32(IMFMediaType *media_type, const GUID *key, UINT32 *value,
+        UINT32 default_value)
+{
+    if (FAILED(IMFMediaType_GetUINT32(media_type, key, value)))
+        *value = default_value;
+}
+
+static void mf_get_attribute_uint64(IMFMediaType *media_type, const GUID *key, UINT64 *value,
+        UINT64 default_value)
+{
+    if (FAILED(IMFMediaType_GetUINT64(media_type, key, value)))
+        *value = default_value;
 }
 
 static HRESULT video_mixer_collect_output_types(struct video_mixer *mixer, const DXVA2_VideoDesc *video_desc,
         IMFMediaType *media_type, IDirectXVideoProcessorService *service, unsigned int device_count,
         const GUID *devices, unsigned int flags)
 {
-    unsigned int i, j, format_count, count;
     struct rt_format *rt_formats = NULL, *ptr;
+    unsigned int i, j, format_count, count;
     HRESULT hr = MF_E_INVALIDMEDIATYPE;
     D3DFORMAT *formats;
     GUID subtype;
@@ -677,11 +755,7 @@ static HRESULT video_mixer_collect_output_types(struct video_mixer *mixer, const
             rt_formats = ptr;
 
             for (j = 0; j < format_count; ++j)
-            {
-                rt_formats[count + j].format = formats[j];
-                rt_formats[count + j].device = devices[i];
-            }
-            count += format_count;
+                video_mixer_append_rt_format(rt_formats, &count, &devices[i], formats[j]);
 
             CoTaskMemFree(formats);
         }
@@ -689,41 +763,48 @@ static HRESULT video_mixer_collect_output_types(struct video_mixer *mixer, const
 
     if (count && !(flags & MFT_SET_TYPE_TEST_ONLY))
     {
-        qsort(rt_formats, count, sizeof(*rt_formats), rt_formats_sort_compare);
+        UINT32 fixed_samples, interlace_mode;
+        MFVideoArea aperture;
+        UINT64 par;
 
-        j = 0;
-        for (i = j + 1; i < count; ++i)
+        if (!(mixer->output.rt_formats = calloc(count, sizeof(*mixer->output.rt_formats))))
         {
-            if (rt_formats[i].format != rt_formats[j].format)
-            {
-                rt_formats[++j] = rt_formats[i];
-            }
+            free(rt_formats);
+            return E_OUTOFMEMORY;
         }
-        count = j + 1;
 
         memcpy(&subtype, &MFVideoFormat_Base, sizeof(subtype));
-        if ((mixer->output.rt_formats = calloc(count, sizeof(*mixer->output.rt_formats))))
+        memset(&aperture, 0, sizeof(aperture));
+        if (FAILED(IMFMediaType_GetBlob(media_type, &MF_MT_GEOMETRIC_APERTURE, (UINT8 *)&aperture,
+                sizeof(aperture), NULL)))
         {
-            for (i = 0; i < count; ++i)
-            {
-                IMFMediaType *rt_media_type;
-
-                subtype.Data1 = rt_formats[i].format;
-                mixer->output.rt_formats[i] = rt_formats[i];
-
-                MFCreateMediaType(&rt_media_type);
-                IMFMediaType_CopyAllItems(media_type, (IMFAttributes *)rt_media_type);
-                IMFMediaType_SetGUID(rt_media_type, &MF_MT_SUBTYPE, &subtype);
-
-                mixer->output.rt_formats[i].media_type = rt_media_type;
-            }
-            mixer->output.rt_formats_count = count;
+            aperture.Area.cx = video_desc->SampleWidth;
+            aperture.Area.cy = video_desc->SampleHeight;
         }
-        else
+        interlace_mode = video_mixer_get_interlace_mode_from_video_desc(video_desc);
+        mf_get_attribute_uint64(media_type, &MF_MT_PIXEL_ASPECT_RATIO, &par, (UINT64)1 << 32 | 1);
+        mf_get_attribute_uint32(media_type, &MF_MT_FIXED_SIZE_SAMPLES, &fixed_samples, 1);
+
+        for (i = 0; i < count; ++i)
         {
-            hr = E_OUTOFMEMORY;
-            count = 0;
+            IMFMediaType *rt_media_type;
+
+            subtype.Data1 = rt_formats[i].format;
+            mixer->output.rt_formats[i] = rt_formats[i];
+
+            MFCreateMediaType(&rt_media_type);
+            IMFMediaType_CopyAllItems(media_type, (IMFAttributes *)rt_media_type);
+            IMFMediaType_SetGUID(rt_media_type, &MF_MT_SUBTYPE, &subtype);
+            IMFMediaType_SetUINT64(rt_media_type, &MF_MT_FRAME_SIZE, (UINT64)aperture.Area.cx << 32 | aperture.Area.cy);
+            IMFMediaType_SetBlob(rt_media_type, &MF_MT_GEOMETRIC_APERTURE, (const UINT8 *)&aperture, sizeof(aperture));
+            IMFMediaType_SetBlob(rt_media_type, &MF_MT_MINIMUM_DISPLAY_APERTURE, (const UINT8 *)&aperture, sizeof(aperture));
+            IMFMediaType_SetUINT32(rt_media_type, &MF_MT_INTERLACE_MODE, interlace_mode);
+            IMFMediaType_SetUINT64(rt_media_type, &MF_MT_PIXEL_ASPECT_RATIO, par);
+            IMFMediaType_SetUINT32(rt_media_type, &MF_MT_FIXED_SIZE_SAMPLES, fixed_samples);
+
+            mixer->output.rt_formats[i].media_type = rt_media_type;
         }
+        mixer->output.rt_formats_count = count;
     }
 
     free(rt_formats);
@@ -798,6 +879,8 @@ static HRESULT WINAPI video_mixer_transform_SetInputType(IMFTransform *iface, DW
                             if (mixer->inputs[0].media_type)
                                 IMFMediaType_Release(mixer->inputs[0].media_type);
                             mixer->inputs[0].media_type = media_type;
+                            mixer->inputs[0].frame_size.cx = video_desc.SampleWidth;
+                            mixer->inputs[0].frame_size.cy = video_desc.SampleHeight;
                             IMFMediaType_AddRef(mixer->inputs[0].media_type);
                         }
                         CoTaskMemFree(guids);
@@ -820,17 +903,23 @@ static HRESULT WINAPI video_mixer_transform_SetInputType(IMFTransform *iface, DW
 
 static HRESULT WINAPI video_mixer_transform_SetOutputType(IMFTransform *iface, DWORD id, IMFMediaType *type, DWORD flags)
 {
-    const unsigned int equality_flags = MF_MEDIATYPE_EQUAL_MAJOR_TYPES |
-            MF_MEDIATYPE_EQUAL_FORMAT_TYPES | MF_MEDIATYPE_EQUAL_FORMAT_DATA;
+    const unsigned int equality_flags = MF_MEDIATYPE_EQUAL_MAJOR_TYPES | MF_MEDIATYPE_EQUAL_FORMAT_TYPES;
     struct video_mixer *mixer = impl_from_IMFTransform(iface);
     HRESULT hr = MF_E_INVALIDMEDIATYPE;
     unsigned int i, compare_flags;
+    BOOL is_compressed = TRUE;
 
     TRACE("%p, %u, %p, %#x.\n", iface, id, type, flags);
 
     if (id)
         return MF_E_INVALIDSTREAMNUMBER;
 
+    if (!type)
+        return E_INVALIDARG;
+
+    if (FAILED(IMFMediaType_IsCompressedFormat(type, &is_compressed)) || is_compressed)
+        return MF_E_INVALIDMEDIATYPE;
+
     EnterCriticalSection(&mixer->cs);
 
     for (i = 0; i < mixer->output.rt_formats_count; ++i)
@@ -865,8 +954,13 @@ static HRESULT WINAPI video_mixer_transform_SetOutputType(IMFTransform *iface, D
             rt_format = subtype.Data1;
 
             if (SUCCEEDED(hr = IDirectXVideoProcessorService_CreateVideoProcessor(service, &mixer->output.rt_formats[i].device,
-                    &video_desc, rt_format, MAX_MIXER_INPUT_STREAMS, &mixer->processor)))
+                    &video_desc, rt_format, MAX_MIXER_INPUT_SUBSTREAMS, &mixer->processor)))
             {
+                if (FAILED(IMFMediaType_GetBlob(type, &MF_MT_GEOMETRIC_APERTURE, (UINT8 *)&mixer->aperture,
+                        sizeof(mixer->aperture), NULL)))
+                {
+                    memset(&mixer->aperture, 0, sizeof(mixer->aperture));
+                }
                 if (mixer->output.media_type)
                     IMFMediaType_Release(mixer->output.media_type);
                 mixer->output.media_type = type;
@@ -1029,46 +1123,29 @@ static HRESULT WINAPI video_mixer_transform_ProcessMessage(IMFTransform *iface,
     HRESULT hr = S_OK;
     unsigned int i;
 
-    TRACE("%p, %u, %#lx.\n", iface, message, param);
+    TRACE("%p, %#x, %#lx.\n", iface, message, param);
+
+    EnterCriticalSection(&mixer->cs);
 
     switch (message)
     {
         case MFT_MESSAGE_SET_D3D_MANAGER:
-
-            EnterCriticalSection(&mixer->cs);
-
             video_mixer_release_device_manager(mixer);
             if (param)
                 hr = IUnknown_QueryInterface((IUnknown *)param, &IID_IDirect3DDeviceManager9, (void **)&mixer->device_manager);
 
-            LeaveCriticalSection(&mixer->cs);
-
             break;
 
         case MFT_MESSAGE_COMMAND_FLUSH:
-
-            EnterCriticalSection(&mixer->cs);
-
-            for (i = 0; i < mixer->input_count; ++i)
-            {
-                if (mixer->inputs[i].sample)
-                {
-                    IMFSample_Release(mixer->inputs[i].sample);
-                    mixer->inputs[i].sample = NULL;
-                    mixer->inputs[i].sample_requested = 0;
-                }
-            }
-
-            LeaveCriticalSection(&mixer->cs);
+            video_mixer_flush_input(mixer);
 
             break;
 
         case MFT_MESSAGE_NOTIFY_BEGIN_STREAMING:
         case MFT_MESSAGE_NOTIFY_END_STREAMING:
-
-            EnterCriticalSection(&mixer->cs);
-
-            if (!mixer->is_streaming)
+            if (mixer->is_streaming)
+                video_mixer_flush_input(mixer);
+            else
             {
                 for (i = 0; i < mixer->input_count; ++i)
                     video_mixer_request_sample(mixer, i);
@@ -1076,8 +1153,6 @@ static HRESULT WINAPI video_mixer_transform_ProcessMessage(IMFTransform *iface,
 
             mixer->is_streaming = message == MFT_MESSAGE_NOTIFY_BEGIN_STREAMING;
 
-            LeaveCriticalSection(&mixer->cs);
-
             break;
 
         case MFT_MESSAGE_COMMAND_DRAIN:
@@ -1088,6 +1163,8 @@ static HRESULT WINAPI video_mixer_transform_ProcessMessage(IMFTransform *iface,
             hr = E_NOTIMPL;
     }
 
+    LeaveCriticalSection(&mixer->cs);
+
     return hr;
 }
 
@@ -1108,10 +1185,12 @@ static HRESULT WINAPI video_mixer_transform_ProcessInput(IMFTransform *iface, DW
     {
         if (!input->media_type || !mixer->output.media_type)
             hr = MF_E_TRANSFORM_TYPE_NOT_SET;
-        else if (input->sample)
+        else if (input->sample && !mixer->output_rendered)
             hr = MF_E_NOTACCEPTING;
         else
         {
+            if (input->sample && mixer->output_rendered)
+                video_mixer_flush_input(mixer);
             mixer->is_streaming = 1;
             input->sample_requested = 0;
             input->sample = sample;
@@ -1162,33 +1241,59 @@ static HRESULT video_mixer_get_d3d_device(struct video_mixer *mixer, IDirect3DDe
     return hr;
 }
 
+static BOOL video_mixer_rect_needs_scaling(const MFVideoNormalizedRect *scale)
+{
+    return scale->left != 0.0f || scale->top != 0.0f || scale->right != 1.0f || scale->bottom != 1.0f;
+}
+
 static void video_mixer_scale_rect(RECT *rect, unsigned int width, unsigned int height,
         const MFVideoNormalizedRect *scale)
 {
-    if (rect->left == 0.0f && rect->top == 0.0f && rect->right == 1.0f && rect->bottom == 1.0f)
-    {
-        SetRect(rect, 0, 0, width, height);
-    }
-    else
+    if (video_mixer_rect_needs_scaling(scale))
     {
         rect->left = width * scale->left;
         rect->right = width * scale->right;
         rect->top = height * scale->top;
         rect->bottom = height * scale->bottom;
     }
+    else
+        SetRect(rect, 0, 0, width, height);
+}
+
+static void video_mixer_correct_aspect_ratio(const RECT *src, RECT *dst)
+{
+   unsigned int src_width = src->right - src->left, src_height = src->bottom - src->top;
+   unsigned int dst_width = dst->right - dst->left, dst_height = dst->bottom - dst->top;
+
+   if (src_width * dst_height > dst_width * src_height)
+   {
+       /* src is "wider" than dst. */
+       unsigned int dst_center = (dst->top + dst->bottom) / 2;
+       unsigned int scaled_height = src_height * dst_width / src_width;
+
+       dst->top = dst_center - scaled_height / 2;
+       dst->bottom = dst->top + scaled_height;
+   }
+   else if (src_width * dst_height < dst_width * src_height)
+   {
+       /* src is "taller" than dst. */
+       unsigned int dst_center = (dst->left + dst->right) / 2;
+       unsigned int scaled_width = src_width * dst_height / src_height;
+
+       dst->left = dst_center - scaled_width / 2;
+       dst->right = dst->left + scaled_width;
+   }
 }
 
 static void video_mixer_render(struct video_mixer *mixer, IDirect3DSurface9 *rt)
 {
-    DXVA2_VideoSample samples[1] = {{ 0 }};
+    DXVA2_VideoSample samples[MAX_MIXER_INPUT_STREAMS] = {{ 0 }};
     DXVA2_VideoProcessBltParams params = { 0 };
     MFVideoNormalizedRect zoom_rect;
     struct input_stream *stream;
-    IDirect3DSurface9 *surface;
-    D3DSURFACE_DESC desc;
-    HRESULT hr;
-
-    IDirect3DSurface9_GetDesc(rt, &desc);
+    HRESULT hr = S_OK;
+    unsigned int i;
+    RECT dst;
 
     if (FAILED(IMFAttributes_GetBlob(mixer->attributes, &VIDEO_ZOOM_RECT, (UINT8 *)&zoom_rect,
             sizeof(zoom_rect), NULL)))
@@ -1197,28 +1302,56 @@ static void video_mixer_render(struct video_mixer *mixer, IDirect3DSurface9 *rt)
         zoom_rect.right = zoom_rect.bottom = 1.0f;
     }
 
-    video_mixer_scale_rect(&params.TargetRect, desc.Width, desc.Height, &zoom_rect);
+    SetRect(&dst, 0, 0, mixer->aperture.Area.cx, mixer->aperture.Area.cy);
+    OffsetRect(&dst, mixer->aperture.OffsetX.value, mixer->aperture.OffsetY.value);
 
-    /* FIXME: for now only handle reference stream. */
-
-    video_mixer_get_input(mixer, 0, &stream);
-
-    if (FAILED(hr = video_mixer_get_sample_surface(stream->sample, &surface)))
+    for (i = 0; i < mixer->input_count; ++i)
     {
-        WARN("Failed to get source surface, hr %#x.\n", hr);
-        return;
+        DXVA2_VideoSample *sample = &samples[i];
+        IDirect3DSurface9 *surface;
+
+        stream = mixer->zorder[i];
+
+        if (FAILED(hr = video_mixer_get_sample_surface(stream->sample, &surface)))
+        {
+            WARN("Failed to get source surface for stream %u, hr %#x.\n", i, hr);
+            break;
+        }
+
+        /* Full input frame corrected to full destination rectangle. */
+
+        video_mixer_scale_rect(&sample->SrcRect, stream->frame_size.cx, stream->frame_size.cy, &zoom_rect);
+        CopyRect(&sample->DstRect, &dst);
+        video_mixer_correct_aspect_ratio(&sample->SrcRect, &sample->DstRect);
+
+        if (video_mixer_rect_needs_scaling(&stream->rect))
+            WARN("Ignoring stream %u rectangle %s.\n", stream->id, debugstr_normalized_rect(&stream->rect));
+
+        sample->SampleFormat.SampleFormat = stream->id == 0 ? DXVA2_SampleProgressiveFrame : DXVA2_SampleSubStream;
+        sample->SrcSurface = surface;
+        sample->PlanarAlpha = DXVA2_Fixed32OpaqueAlpha();
     }
 
-    IDirect3DSurface9_GetDesc(surface, &desc);
+    if (SUCCEEDED(hr))
+    {
+        SetRect(&params.TargetRect, 0, 0, mixer->aperture.Area.cx, mixer->aperture.Area.cy);
+        OffsetRect(&params.TargetRect, mixer->aperture.OffsetX.value, mixer->aperture.OffsetY.value);
 
-    samples[0].SrcSurface = surface;
-    SetRect(&samples[0].SrcRect, 0, 0, desc.Width, desc.Height);
-    video_mixer_scale_rect(&samples[0].DstRect, desc.Width, desc.Height, &stream->rect);
+        params.BackgroundColor = mixer->bkgnd_color.ayuv;
+        params.Alpha = DXVA2_Fixed32OpaqueAlpha();
 
-    if (FAILED(hr = IDirectXVideoProcessor_VideoProcessBlt(mixer->processor, rt, &params, samples, 1, NULL)))
-        WARN("Failed to process samples, hr %#x.\n", hr);
+        if (FAILED(hr = IDirectXVideoProcessor_VideoProcessBlt(mixer->processor, rt, &params, samples,
+                mixer->input_count, NULL)))
+        {
+            WARN("Failed to process samples, hr %#x.\n", hr);
+        }
+    }
 
-    IDirect3DSurface9_Release(surface);
+    for (i = 0; i < mixer->input_count; ++i)
+    {
+        if (samples[i].SrcSurface)
+            IDirect3DSurface9_Release(samples[i].SrcSurface);
+    }
 }
 
 static HRESULT video_mixer_get_sample_desired_time(IMFSample *sample, LONGLONG *timestamp, LONGLONG *duration)
@@ -1226,6 +1359,7 @@ static HRESULT video_mixer_get_sample_desired_time(IMFSample *sample, LONGLONG *
     IMFDesiredSample *desired;
     HRESULT hr;
 
+    *timestamp = *duration = 0;
     if (SUCCEEDED(hr = IMFSample_QueryInterface(sample, &IID_IMFDesiredSample, (void **)&desired)))
     {
         hr = IMFDesiredSample_GetDesiredSampleTimeAndDuration(desired, timestamp, duration);
@@ -1235,6 +1369,18 @@ static HRESULT video_mixer_get_sample_desired_time(IMFSample *sample, LONGLONG *
     return hr;
 }
 
+static BOOL video_mixer_has_input(const struct video_mixer *mixer)
+{
+    unsigned int i;
+
+    for (i = 0; i < mixer->input_count; ++i)
+    {
+        if (!mixer->inputs[i].sample) return FALSE;
+    }
+
+    return TRUE;
+}
+
 static HRESULT WINAPI video_mixer_transform_ProcessOutput(IMFTransform *iface, DWORD flags, DWORD count,
         MFT_OUTPUT_DATA_BUFFER *buffers, DWORD *status)
 {
@@ -1242,6 +1388,7 @@ static HRESULT WINAPI video_mixer_transform_ProcessOutput(IMFTransform *iface, D
     LONGLONG timestamp, duration;
     IDirect3DSurface9 *surface;
     IDirect3DDevice9 *device;
+    BOOL repaint = FALSE;
     unsigned int i;
     HRESULT hr;
 
@@ -1261,15 +1408,20 @@ static HRESULT WINAPI video_mixer_transform_ProcessOutput(IMFTransform *iface, D
     {
         if (mixer->is_streaming)
         {
-            for (i = 0; i < mixer->input_count; ++i)
+            /* Desired timestamp is ignored, duration is required to be non-zero but is not used either. */
+            if (SUCCEEDED(video_mixer_get_sample_desired_time(buffers->pSample, &timestamp, &duration)))
             {
-                if (!mixer->inputs[i].sample)
+                if (!(repaint = !!duration))
                 {
-                    hr = MF_E_TRANSFORM_NEED_MORE_INPUT;
-                    break;
+                    WARN("Unexpected sample duration.\n");
+                    hr = E_INVALIDARG;
                 }
             }
 
+            /* Not enough input, or no new input. */
+            if (SUCCEEDED(hr) && (!video_mixer_has_input(mixer) || (!repaint && mixer->output_rendered)))
+                hr = MF_E_TRANSFORM_NEED_MORE_INPUT;
+
             if (SUCCEEDED(hr))
             {
                 video_mixer_render(mixer, surface);
@@ -1277,22 +1429,17 @@ static HRESULT WINAPI video_mixer_transform_ProcessOutput(IMFTransform *iface, D
                 timestamp = duration = 0;
                 if (SUCCEEDED(IMFSample_GetSampleTime(mixer->inputs[0].sample, &timestamp)))
                 {
-                    IMFSample_SetSampleTime(buffers->pSample, timestamp);
-
                     IMFSample_GetSampleDuration(mixer->inputs[0].sample, &duration);
+                    IMFSample_SetSampleTime(buffers->pSample, timestamp);
                     IMFSample_SetSampleDuration(buffers->pSample, duration);
                 }
+                mixer->output_rendered = 1;
             }
 
-            if (SUCCEEDED(hr))
+            if (SUCCEEDED(hr) && !repaint)
             {
                 for (i = 0; i < mixer->input_count; ++i)
-                {
-                    if (mixer->inputs[i].sample)
-                        IMFSample_Release(mixer->inputs[i].sample);
-                    mixer->inputs[i].sample = NULL;
                     video_mixer_request_sample(mixer, i);
-                }
             }
         }
         else
@@ -1495,8 +1642,11 @@ static HRESULT WINAPI video_mixer_control_SetStreamZOrder(IMFVideoMixerControl2
         /* Lowest zorder only applies to reference stream. */
         if (id && !zorder)
             hr = MF_E_INVALIDREQUEST;
-        else
+        else if (stream->zorder != zorder)
+        {
             stream->zorder = zorder;
+            video_mixer_update_zorder_map(mixer);
+        }
     }
 
     LeaveCriticalSection(&mixer->cs);
@@ -1532,7 +1682,7 @@ static HRESULT WINAPI video_mixer_control_SetStreamOutputRect(IMFVideoMixerContr
     struct input_stream *stream;
     HRESULT hr;
 
-    TRACE("%p, %u, %p.\n", iface, id, rect);
+    TRACE("%p, %u, %s.\n", iface, id, debugstr_normalized_rect(rect));
 
     if (!rect)
         return E_POINTER;
@@ -1858,12 +2008,28 @@ static HRESULT WINAPI video_mixer_processor_GetBackgroundColor(IMFVideoProcessor
         return E_POINTER;
 
     EnterCriticalSection(&mixer->cs);
-    *color = mixer->bkgnd_color;
+    *color = mixer->bkgnd_color.rgba;
     LeaveCriticalSection(&mixer->cs);
 
     return S_OK;
 }
 
+static void video_mixer_rgb_to_ycbcr(COLORREF rgb, DXVA2_AYUVSample16 *ayuv)
+{
+    int y, cb, cr, r, g, b;
+
+    r = GetRValue(rgb); g = GetGValue(rgb); b = GetBValue(rgb);
+    /* Coefficients according to SDTV ITU-R BT.601 */
+    y  = (77 * r + 150 * g +  29 * b + 128) / 256 + 16;
+    cb = (-44 * r - 87 * g + 131 * b + 128) / 256 + 128;
+    cr = (131 * r - 110 * g - 21 * b + 128) / 256 + 128;
+
+    ayuv->Y  = y * 0x100;
+    ayuv->Cb = cb * 0x100;
+    ayuv->Cr = cr * 0x100;
+    ayuv->Alpha = 0xffff;
+}
+
 static HRESULT WINAPI video_mixer_processor_SetBackgroundColor(IMFVideoProcessor *iface, COLORREF color)
 {
     struct video_mixer *mixer = impl_from_IMFVideoProcessor(iface);
@@ -1871,7 +2037,11 @@ static HRESULT WINAPI video_mixer_processor_SetBackgroundColor(IMFVideoProcessor
     TRACE("%p, %#x.\n", iface, color);
 
     EnterCriticalSection(&mixer->cs);
-    mixer->bkgnd_color = color;
+    if (mixer->bkgnd_color.rgba != color)
+    {
+        video_mixer_rgb_to_ycbcr(color, &mixer->bkgnd_color.ayuv);
+        mixer->bkgnd_color.rgba = color;
+    }
     LeaveCriticalSection(&mixer->cs);
 
     return S_OK;
@@ -2402,6 +2572,8 @@ HRESULT evr_mixer_create(IUnknown *outer, void **out)
     object->lower_bound = MFT_OUTPUT_BOUND_LOWER_UNBOUNDED;
     object->upper_bound = MFT_OUTPUT_BOUND_UPPER_UNBOUNDED;
     video_mixer_init_input(&object->inputs[0]);
+    video_mixer_update_zorder_map(object);
+    video_mixer_rgb_to_ycbcr(object->bkgnd_color.rgba, &object->bkgnd_color.ayuv);
     InitializeCriticalSection(&object->cs);
     if (FAILED(hr = MFCreateAttributes(&object->attributes, 0)))
     {
