@@ -461,7 +461,8 @@ static HRESULT prop_put(jsdisp_t *This, dispex_prop_t *prop, jsval_t val)
             prop_iter = prototype_iter->props + prop_iter->u.ref;
         } while(prop_iter->type == PROP_PROTREF);
 
-        if(prop_iter->type == PROP_ACCESSOR)
+        if(prop_iter->type == PROP_ACCESSOR ||
+           (prop_iter->type == PROP_BUILTIN && prop_iter->u.p->setter))
             prop = prop_iter;
     }
 
@@ -2028,6 +2029,7 @@ HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, WORD flags, uns
     if(jsdisp && jsdisp->ctx == ctx) {
         if(flags & DISPATCH_PROPERTYPUT) {
             FIXME("disp_call(propput) on builtin object\n");
+            jsdisp_release(jsdisp);
             return E_FAIL;
         }
 
@@ -2037,6 +2039,8 @@ HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, WORD flags, uns
         jsdisp_release(jsdisp);
         return hres;
     }
+    if(jsdisp)
+        jsdisp_release(jsdisp);
 
     flags &= ~DISPATCH_JSCRIPT_INTERNAL_MASK;
     if(ret && argc)
@@ -2106,6 +2110,8 @@ HRESULT disp_call_value(script_ctx_t *ctx, IDispatch *disp, IDispatch *jsthis, W
         jsdisp_release(jsdisp);
         return hres;
     }
+    if(jsdisp)
+        jsdisp_release(jsdisp);
 
     flags &= ~DISPATCH_JSCRIPT_INTERNAL_MASK;
     if(r && argc && flags == DISPATCH_METHOD)
@@ -2205,6 +2211,8 @@ HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, jsval_t val)
         VARIANT var;
         DISPPARAMS dp  = {&var, &dispid, 1, 1};
 
+        if(jsdisp)
+            jsdisp_release(jsdisp);
         hres = jsval_to_variant(val, &var);
         if(FAILED(hres))
             return hres;
@@ -2230,6 +2238,8 @@ HRESULT disp_propput_name(script_ctx_t *ctx, IDispatch *disp, const WCHAR *name,
         BSTR str;
         DISPID id;
 
+        if(jsdisp)
+            jsdisp_release(jsdisp);
         if(!(str = SysAllocString(name)))
             return E_OUTOFMEMORY;
 
@@ -2248,7 +2258,9 @@ HRESULT disp_propput_name(script_ctx_t *ctx, IDispatch *disp, const WCHAR *name,
         return disp_propput(ctx, disp, id, val);
     }
 
-    return jsdisp_propput_name(jsdisp, name, val);
+    hres = jsdisp_propput_name(jsdisp, name, val);
+    jsdisp_release(jsdisp);
+    return hres;
 }
 
 HRESULT jsdisp_propget_name(jsdisp_t *obj, const WCHAR *name, jsval_t *val)
@@ -2312,6 +2324,8 @@ HRESULT disp_propget(script_ctx_t *ctx, IDispatch *disp, DISPID id, jsval_t *val
         jsdisp_release(jsdisp);
         return hres;
     }
+    if(jsdisp)
+        jsdisp_release(jsdisp);
 
     V_VT(&var) = VT_EMPTY;
     hres = disp_invoke(ctx, disp, id, INVOKE_PROPERTYGET, &dp, &var);
@@ -2641,6 +2655,26 @@ HRESULT jsdisp_define_data_property(jsdisp_t *obj, const WCHAR *name, unsigned f
     return jsdisp_define_property(obj, name, &prop_desc);
 }
 
+HRESULT jsdisp_change_prototype(jsdisp_t *obj, jsdisp_t *proto)
+{
+    DWORD i;
+
+    if(obj->prototype == proto)
+        return S_OK;
+
+    if(obj->prototype) {
+        for(i = 0; i < obj->prop_cnt; i++)
+            if(obj->props[i].type == PROP_PROTREF)
+                obj->props[i].type = PROP_DELETED;
+        jsdisp_release(obj->prototype);
+    }
+
+    obj->prototype = proto;
+    if(proto)
+        jsdisp_addref(proto);
+    return S_OK;
+}
+
 void jsdisp_freeze(jsdisp_t *obj, BOOL seal)
 {
     unsigned int i;
