@@ -29,26 +29,26 @@
 static DWORD WINAPIV doit(DWORD flags, LPCVOID src, DWORD msg_id, DWORD lang_id,
                           LPSTR out, DWORD outsize, ... )
 {
-    __ms_va_list list;
+    va_list list;
     DWORD r;
 
-    __ms_va_start(list, outsize);
+    va_start(list, outsize);
     r = FormatMessageA(flags, src, msg_id,
         lang_id, out, outsize, &list);
-    __ms_va_end(list);
+    va_end(list);
     return r;
 }
 
 static DWORD WINAPIV doitW(DWORD flags, LPCVOID src, DWORD msg_id, DWORD lang_id,
                            LPWSTR out, DWORD outsize, ... )
 {
-    __ms_va_list list;
+    va_list list;
     DWORD r;
 
-    __ms_va_start(list, outsize);
+    va_start(list, outsize);
     r = FormatMessageW(flags, src, msg_id,
         lang_id, out, outsize, &list);
-    __ms_va_end(list);
+    va_end(list);
     return r;
 }
 
@@ -144,7 +144,7 @@ static void test_message_from_string_wide(void)
     ok(!lstrcmpW(L"test", out), "failed out=%s\n", wine_dbgstr_w(out));
     ok(r==4, "failed: r=%d\n", r);
 
-    /* S is ascii */
+    /* S is ansi */
     r = doitW(FORMAT_MESSAGE_FROM_STRING, L"%1!S!", 0, 0, out, ARRAY_SIZE(out), "test");
     ok(!lstrcmpW(L"test", out), "failed out=%s\n", wine_dbgstr_w(out));
     ok(r==4, "failed: r=%d\n", r);
@@ -297,11 +297,11 @@ static void test_message_from_string_wide(void)
     {
         ULONG_PTR args[] = { 6, 4, 2, 5, 3, 1 };
         r = FormatMessageW(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY, L"%1!*.*u!,%1!*.*u!",
-                           0, 0, out, ARRAY_SIZE(out), (__ms_va_list *)args );
+                           0, 0, out, ARRAY_SIZE(out), (va_list *)args );
         ok(!lstrcmpW(L"  0002, 00003", out),"failed out=[%s]\n", wine_dbgstr_w(out));
         ok(r==13,"failed: r=%d\n",r);
         r = FormatMessageW(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY, L"%1!*.*u!,%4!*.*u!",
-                           0, 0, out, ARRAY_SIZE(out), (__ms_va_list *)args );
+                           0, 0, out, ARRAY_SIZE(out), (va_list *)args );
         ok(!lstrcmpW(L"  0002,  001", out),"failed out=[%s]\n", wine_dbgstr_w(out));
         ok(r==12,"failed: r=%d\n",r);
     }
@@ -417,7 +417,7 @@ static void test_message_from_string(void)
     ok(!strcmp("test", out),"failed out=[%s]\n",out);
     ok(r==4,"failed: r=%d\n",r);
 
-    /* s is ascii */
+    /* s is ansi */
     r = doit(FORMAT_MESSAGE_FROM_STRING, "%1!s!", 0, 0, out, ARRAY_SIZE(out), "test");
     ok(!strcmp("test", out),"failed out=[%s]\n",out);
     ok(r==4,"failed: r=%d\n",r);
@@ -593,11 +593,11 @@ static void test_message_from_string(void)
         {
             ULONG_PTR args[] = { 6, 4, 2, 5, 3, 1 };
             r = FormatMessageA(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY,
-                               "%1!*.*u!,%1!*.*u!", 0, 0, out, sizeof(out), (__ms_va_list *)args );
+                               "%1!*.*u!,%1!*.*u!", 0, 0, out, sizeof(out), (va_list *)args );
             ok(!strcmp("  0002, 00003", out),"failed out=[%s]\n",out);
             ok(r==13,"failed: r=%d\n",r);
             r = FormatMessageA(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ARGUMENT_ARRAY,
-                               "%1!*.*u!,%4!*.*u!", 0, 0, out, sizeof(out), (__ms_va_list *)args );
+                               "%1!*.*u!,%4!*.*u!", 0, 0, out, sizeof(out), (va_list *)args );
             ok(!strcmp("  0002,  001", out),"failed out=[%s]\n",out);
             ok(r==12,"failed: r=%d\n",r);
         }
