@@ -25,9 +25,11 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/stat.h>
 #ifdef HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
 #endif
@@ -1696,61 +1698,135 @@ static enum mountmgr_fs_type get_mountmgr_fs_type(enum fs_type fs_type)
 }
 
 /* query information about an existing dos drive, by letter or udi */
-NTSTATUS query_dos_device( int letter, enum device_type *type, enum mountmgr_fs_type *fs_type,
-                           DWORD *serial, char **device, char **mount_point, WCHAR **label )
+static struct volume *find_volume_by_letter( int letter )
 {
-    NTSTATUS status = STATUS_NO_SUCH_DEVICE;
+    struct volume *volume = NULL;
     struct dos_drive *drive;
-    struct disk_device *disk_device;
 
-    EnterCriticalSection( &device_section );
     LIST_FOR_EACH_ENTRY( drive, &drives_list, struct dos_drive, entry )
     {
         if (drive->drive != letter) continue;
-        disk_device = drive->volume->device;
-        if (type) *type = disk_device->type;
-        if (fs_type) *fs_type = get_mountmgr_fs_type( drive->volume->fs_type );
-        if (serial) *serial = drive->volume->serial;
-        if (device) *device = strdupA( disk_device->unix_device );
-        if (mount_point) *mount_point = strdupA( disk_device->unix_mount );
-        if (label) *label = strdupW( drive->volume->label );
-        status = STATUS_SUCCESS;
+        volume = grab_volume( drive->volume );
+        TRACE( "found matching volume %s for drive letter %c:\n", debugstr_guid(&volume->guid),
+               'a' + letter );
         break;
     }
-    LeaveCriticalSection( &device_section );
-    return status;
+    return volume;
 }
 
 /* query information about an existing unix device, by dev_t */
-NTSTATUS query_unix_device( ULONGLONG unix_dev, enum device_type *type,
-                            enum mountmgr_fs_type *fs_type, DWORD *serial, char **device,
-                            char **mount_point, WCHAR **label )
+static struct volume *find_volume_by_unixdev( ULONGLONG unix_dev )
 {
-    NTSTATUS status = STATUS_NO_SUCH_DEVICE;
     struct volume *volume;
-    struct disk_device *disk_device;
     struct stat st;
 
-    EnterCriticalSection( &device_section );
     LIST_FOR_EACH_ENTRY( volume, &volumes_list, struct volume, entry )
     {
-        disk_device = volume->device;
-
-        if (!disk_device->unix_device
-            || stat( disk_device->unix_device, &st ) < 0
+        if (!volume->device->unix_device || stat( volume->device->unix_device, &st ) < 0
             || st.st_rdev != unix_dev)
             continue;
 
-        if (type) *type = disk_device->type;
-        if (fs_type) *fs_type = get_mountmgr_fs_type( volume->fs_type );
-        if (serial) *serial = volume->serial;
-        if (device) *device = strdupA( disk_device->unix_device );
-        if (mount_point) *mount_point = strdupA( disk_device->unix_mount );
-        if (label) *label = strdupW( volume->label );
-        status = STATUS_SUCCESS;
-        break;
+        TRACE( "found matching volume %s\n", debugstr_guid(&volume->guid) );
+        return grab_volume( volume );
+    }
+    return NULL;
+}
+
+/* implementation of IOCTL_MOUNTMGR_QUERY_UNIX_DRIVE */
+NTSTATUS query_unix_drive( void *buff, SIZE_T insize, SIZE_T outsize, IO_STATUS_BLOCK *iosb )
+{
+    const struct mountmgr_unix_drive *input = buff;
+    struct mountmgr_unix_drive *output = NULL;
+    char *device, *mount_point;
+    int letter = tolowerW( input->letter );
+    DWORD size, type = DEVICE_UNKNOWN, serial;
+    NTSTATUS status = STATUS_SUCCESS;
+    enum mountmgr_fs_type fs_type;
+    enum device_type device_type;
+    struct volume *volume;
+    char *ptr;
+    WCHAR *label;
+
+    if (letter && (letter < 'a' || letter > 'z')) return STATUS_INVALID_PARAMETER;
+
+    EnterCriticalSection( &device_section );
+    if (letter)
+        volume = find_volume_by_letter( letter - 'a' );
+    else
+        volume = find_volume_by_unixdev( input->unix_dev );
+    if (volume)
+    {
+        device_type = volume->device->type;
+        fs_type = get_mountmgr_fs_type( volume->fs_type );
+        serial = volume->serial;
+        device = strdupA( volume->device->unix_device );
+        mount_point = strdupA( volume->device->unix_mount );
+        label = strdupW( volume->label );
+        release_volume( volume );
     }
     LeaveCriticalSection( &device_section );
+
+    if (!volume)
+        return STATUS_NO_SUCH_DEVICE;
+
+    switch (device_type)
+    {
+    case DEVICE_UNKNOWN:      type = DRIVE_UNKNOWN; break;
+    case DEVICE_HARDDISK:     type = DRIVE_REMOVABLE; break;
+    case DEVICE_HARDDISK_VOL: type = DRIVE_FIXED; break;
+    case DEVICE_FLOPPY:       type = DRIVE_REMOVABLE; break;
+    case DEVICE_CDROM:        type = DRIVE_CDROM; break;
+    case DEVICE_DVD:          type = DRIVE_CDROM; break;
+    case DEVICE_NETWORK:      type = DRIVE_REMOTE; break;
+    case DEVICE_RAMDISK:      type = DRIVE_RAMDISK; break;
+    }
+
+    size = sizeof(*output);
+    if (label) size += (strlenW(label) + 1) * sizeof(WCHAR);
+    if (device) size += strlen(device) + 1;
+    if (mount_point) size += strlen(mount_point) + 1;
+
+    input = NULL;
+    output = buff;
+    output->size = size;
+    output->letter = letter;
+    output->type = type;
+    output->fs_type = fs_type;
+    output->serial = serial;
+    output->mount_point_offset = 0;
+    output->device_offset = 0;
+    output->label_offset = 0;
+
+    ptr = (char *)(output + 1);
+
+    if (label && ptr + (strlenW(label) + 1) * sizeof(WCHAR) - (char *)output <= outsize)
+    {
+        output->label_offset = ptr - (char *)output;
+        strcpyW( (WCHAR *)ptr, label );
+        ptr += (strlenW(label) + 1) * sizeof(WCHAR);
+    }
+    if (mount_point && ptr + strlen(mount_point) + 1 - (char *)output <= outsize)
+    {
+        output->mount_point_offset = ptr - (char *)output;
+        strcpy( ptr, mount_point );
+        ptr += strlen(ptr) + 1;
+    }
+    if (device && ptr + strlen(device) + 1 - (char *)output <= outsize)
+    {
+        output->device_offset = ptr - (char *)output;
+        strcpy( ptr, device );
+        ptr += strlen(ptr) + 1;
+    }
+
+    TRACE( "returning %c: dev %s mount %s type %u\n",
+           letter, debugstr_a(device), debugstr_a(mount_point), type );
+
+    iosb->Information = ptr - (char *)output;
+    if (size > outsize) status = STATUS_BUFFER_OVERFLOW;
+
+    RtlFreeHeap( GetProcessHeap(), 0, device );
+    RtlFreeHeap( GetProcessHeap(), 0, mount_point );
+    RtlFreeHeap( GetProcessHeap(), 0, label );
     return status;
 }
 
