@@ -39,8 +39,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msctf);
 
-static HINSTANCE MSCTF_hinstance;
-
 typedef struct
 {
     DWORD id;
@@ -557,7 +555,6 @@ BOOL WINAPI DllMain(HINSTANCE hinst, DWORD fdwReason, LPVOID fImpLoad)
     switch (fdwReason)
     {
         case DLL_PROCESS_ATTACH:
-            MSCTF_hinstance = hinst;
             tlsIndex = TlsAlloc();
             break;
         case DLL_PROCESS_DETACH:
@@ -568,14 +565,6 @@ BOOL WINAPI DllMain(HINSTANCE hinst, DWORD fdwReason, LPVOID fImpLoad)
     return TRUE;
 }
 
-/*************************************************************************
- *              DllCanUnloadNow (MSCTF.@)
- */
-HRESULT WINAPI DllCanUnloadNow(void)
-{
-    return S_FALSE;
-}
-
 /***********************************************************************
  *              DllGetClassObject (MSCTF.@)
  */
@@ -595,22 +584,6 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, LPVOID *ppvOut)
     return CLASS_E_CLASSNOTAVAILABLE;
 }
 
-/***********************************************************************
- *		DllRegisterServer (MSCTF.@)
- */
-HRESULT WINAPI DllRegisterServer(void)
-{
-    return __wine_register_resources( MSCTF_hinstance );
-}
-
-/***********************************************************************
- *		DllUnregisterServer (MSCTF.@)
- */
-HRESULT WINAPI DllUnregisterServer(void)
-{
-    return __wine_unregister_resources( MSCTF_hinstance );
-}
-
 /***********************************************************************
  *              TF_CreateThreadMgr (MSCTF.@)
  */
