@@ -245,6 +245,17 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
     case WM_NCPAINT:
         return NC_HandleNCPaint( hwnd, (HRGN)wParam );
 
+    case WM_NCMOUSEMOVE:
+        {
+            POINT pt;
+            pt.x = (short)LOWORD(lParam);
+            pt.y = (short)HIWORD(lParam);
+            return NC_HandleNCMouseMove( hwnd, pt );
+        }
+
+    case WM_NCMOUSELEAVE:
+        return NC_HandleNCMouseLeave( hwnd );
+
     case WM_NCHITTEST:
         {
             POINT pt;
@@ -496,11 +507,11 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         }
         else if( wParam == VK_F10 )
         {
-            if (GetKeyState(VK_SHIFT) & 0x8000)
+            if (NtUserGetKeyState(VK_SHIFT) & 0x8000)
                 SendMessageW( hwnd, WM_CONTEXTMENU, (WPARAM)hwnd, -1 );
             iF10Key = 1;
         }
-        else if( wParam == VK_ESCAPE && (GetKeyState(VK_SHIFT) & 0x8000))
+        else if (wParam == VK_ESCAPE && (NtUserGetKeyState(VK_SHIFT) & 0x8000))
             SendMessageW( hwnd, WM_SYSCOMMAND, SC_KEYMENU, ' ' );
         break;
 
@@ -723,13 +734,15 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         }
 
     case WM_INPUTLANGCHANGEREQUEST:
-        ActivateKeyboardLayout( (HKL)lParam, 0 );
+        NtUserActivateKeyboardLayout( (HKL)lParam, 0 );
         break;
 
     case WM_INPUTLANGCHANGE:
         {
+            struct user_thread_info *info = get_user_thread_info();
             int count = 0;
             HWND *win_array = WIN_ListChildren( hwnd );
+            info->kbd_layout = (HKL)lParam;
 
             if (!win_array)
                 break;
