@@ -101,18 +101,35 @@ static void * const syscalls[] =
     NtGdiSetVirtualResolution,
     NtGdiSwapBuffers,
     NtGdiTransformPoints,
+    NtUserAddClipboardFormatListener,
+    NtUserAttachThreadInput,
     NtUserCloseDesktop,
     NtUserCloseWindowStation,
     NtUserCreateDesktopEx,
     NtUserCreateWindowStation,
+    NtUserGetClipboardFormatName,
+    NtUserGetClipboardOwner,
+    NtUserGetClipboardSequenceNumber,
+    NtUserGetClipboardViewer,
+    NtUserGetKeyState,
+    NtUserGetKeyboardLayout,
+    NtUserGetKeyboardState,
+    NtUserGetLayeredWindowAttributes,
+    NtUserGetMouseMovePointsEx,
     NtUserGetObjectInformation,
+    NtUserGetOpenClipboardWindow,
     NtUserGetProcessWindowStation,
+    NtUserGetProp,
     NtUserGetThreadDesktop,
     NtUserOpenDesktop,
     NtUserOpenInputDesktop,
     NtUserOpenWindowStation,
+    NtUserRemoveClipboardFormatListener,
+    NtUserRemoveProp,
+    NtUserSetKeyboardState,
     NtUserSetObjectInformation,
     NtUserSetProcessWindowStation,
+    NtUserSetProp,
     NtUserSetThreadDesktop,
 };
 
