@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -27,10 +26,6 @@
 #include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#include <fcntl.h>
 
 #include "build.h"
 
@@ -690,9 +685,9 @@ void output_res_o_file( DLLSPEC *spec )
     strarray_add( &args, res_file );
     strarray_add( &args, "-o" );
     strarray_add( &args, output_file_name );
-    switch (target_cpu)
+    switch (target.cpu)
     {
-        case CPU_x86:
+        case CPU_i386:
             strarray_add( &args, "-F" );
             strarray_add( &args, "pe-i386" );
             break;
